﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.IO;
using System.Net;
using System.Collections;
using System.Web.Script.Serialization;
using System.Reflection;
using Syncfusion.JavaScript;
using Syncfusion.JavaScript.Models;
using System.Text;

namespace XMLImportingHelper
{
    public class Gantt 
    {

        public string Open(GanttImportRequest importRequest)
        {
            Stream respStream;
            string xml = "xml", unSupportedFile = "UnsupportedFile";
            if (importRequest.FileStream != null)
            {
                if (string.IsNullOrEmpty(importRequest.FileType))
                    importRequest.FileType = xml;
            }
            else if (!string.IsNullOrEmpty(importRequest.Url))
            {
                try
                {
                    int read;
                    importRequest.FileStream = new MemoryStream();
                    HttpWebRequest fileReq = (HttpWebRequest)HttpWebRequest.Create(importRequest.Url);
                    HttpWebResponse fileResp = (HttpWebResponse)fileReq.GetResponse();
                    respStream = fileResp.GetResponseStream();
                    byte[] buffer = new byte[fileResp.ContentLength];
                    while ((read = respStream.Read(buffer, 0, buffer.Length)) > 0)
                    {
                        importRequest.FileStream.Write(buffer, 0, read);
                    }
                    importRequest.FileStream.Position = 0;
                    if (string.IsNullOrEmpty(importRequest.FileType))
                        importRequest.FileType = xml;
                }
                catch
                {
                    return "InvalidUrl";
                }
            }
            else if (importRequest.File != null)
            {
                importRequest.FileStream = importRequest.File.InputStream;
                if (string.IsNullOrEmpty(importRequest.FileType))
                    importRequest.FileType = importRequest.File.FileName.Split(new char[] { '.' }).Last();
            }
            else
                return unSupportedFile;

           return GanttImport.ImportFromXML(importRequest);
        }

        public void Save(string ganttModel, string filename)
        {
            GanttExport exportObj = new GanttExport();
            exportObj.FileName = filename;
            exportObj.Export(ConvertGanttObject(ganttModel));
        }

        public void Save(string ganttModel, string filePath, string fileName)
        {
            GanttExport exportObj = new GanttExport();
            exportObj.FilePath = filePath;
            exportObj.FileName = fileName;
            exportObj.LocalSave = true;
            exportObj.Export(ConvertGanttObject(ganttModel));
        }

        private GanttProperties ConvertGanttObject(string ganttModel)
        {
            JavaScriptSerializer serializer = new JavaScriptSerializer();
            serializer.MaxJsonLength = Int32.MaxValue;
            IEnumerable div = (IEnumerable)serializer.Deserialize(ganttModel, typeof(IEnumerable));
            GanttProperties ganttProp = new GanttProperties();
            foreach (KeyValuePair<string, object> ds in div)
            {
                var property = ganttProp.GetType().GetProperty(ds.Key, BindingFlags.Instance | BindingFlags.Public | BindingFlags.IgnoreCase);
                if (property != null)
                {
                    Type type = property.PropertyType;
                    string serialize = serializer.Serialize(ds.Value);
                    object value ;
                    if (ds.Key == "dataSource")
                    {
                        value = serializer.Deserialize<List<BusinessObject>>(serialize);
                    }
                    else if (ds.Key == "resources")
                    {
                        value = serializer.Deserialize<List<GanttResources>>(serialize);
                    }
                    else
                    {
                        value = serializer.Deserialize(serialize, type);
                    }
                    property.SetValue(ganttProp, value, null);
                }
            }
            return ganttProp;
        }
    }
}
